import os
from unittest.mock import MagicMock, patch, call

import pytest

from peek.connection import connect, EsClient, RefreshingEsClient, EsClientManager
from peek.errors import PeekError


def test_connect_default():
    mock_app = MagicMock(name='PeekApp')
    client = connect(mock_app)
    mock_app.input.assert_not_called()
    assert str(client) == 'http://localhost:9200'


def test_connect_will_prompt_password_when_no_password_is_found():
    mock_app = MagicMock(name='PeekApp')
    mock_app.config.as_bool = MagicMock(return_value=False)

    client = connect(mock_app, **{
        'username': 'foo',
        'hosts': 'localhost:9201',
        'use_ssl': True,
    })
    mock_app.input.assert_called()
    assert str(client) == 'foo @ https://localhost:9201'


@patch.dict(os.environ, {'PEEK_PASSWORD': 'password'})
def test_connect_will_not_prompt_password_when_password_is_found_in_env():
    mock_app = MagicMock(name='PeekApp')
    mock_app.config.as_bool = MagicMock(return_value=False)

    client = connect(mock_app, **{
        'username': 'foo',
        'name': 'my-connection'
    })

    mock_app.input.assert_not_called()
    assert str(client) == 'my-connection'


def test_connect_will_prompt_password_when_forced():
    mock_app = MagicMock(name='PeekApp')
    mock_app.config.as_bool = MagicMock(return_value=False)

    connect(mock_app, **{
        'username': 'foo',
        'password': 'password',
        'force_prompt': True,
    })
    mock_app.input.assert_called()


def test_connect_will_fail_when_password_is_not_provided_and_prompt_is_not_allowed():
    mock_app = MagicMock(name='PeekApp')
    mock_app.config.as_bool = MagicMock(return_value=False)

    with pytest.raises(PeekError) as e:
        connect(mock_app, **{
            'username': 'foo',
            'no_prompt': True,
        })
    assert 'Password is not found and password prompt is disabled' in str(e)


def test_connect_will_use_key_ring_when_configured():
    mock_keyring = MagicMock(return_value='password')
    with patch('peek.connection._keyring', mock_keyring):
        mock_app = MagicMock(name='PeekApp')
        mock_app.config.as_bool = MagicMock(return_value=True)

        connect(mock_app, **{
            'username': 'foo',
        })

    mock_keyring.assert_has_calls(
        [call('peek/localhost:9200/userpass', 'foo'),
         call('peek/localhost:9200/userpass', 'foo', 'password')])


def test_connect_has_highest_priority_for_api_key():
    mock_app = MagicMock(name='PeekApp')
    mock_app.config.as_bool = MagicMock(return_value=False)

    client = connect(mock_app, **{
        'api_key': 'id:key',
        'token': 'some-token',
        'username': 'foo',
        'password': 'password',
    })

    assert str(client) == 'K-id @ http://localhost:9200'
    assert client.info()['auth'].startswith('ApiKey id')


def test_connect_has_second_priority_for_token():
    mock_app = MagicMock(name='PeekApp')
    mock_app.config.as_bool = MagicMock(return_value=False)

    client = connect(mock_app, **{
        'token': 'some-token',
        'username': 'foo',
        'password': 'password',
    })

    assert str(client) == 'T-some-token @ http://localhost:9200'
    assert client.info()['auth'].startswith('Token some-token')


def test_connect_will_prefer_cloud_id():
    mock_app = MagicMock(name='PeekApp')
    mock_app.config.as_bool = MagicMock(return_value=False)

    mock_es = MagicMock
    MockEs = MagicMock(return_value=mock_es)

    with patch('peek.connection.Elasticsearch', MockEs):
        client = connect(mock_app, **{
            'username': 'foo',
            'password': 'password',
            'cloud_id': 'my-cloud-id',
            'hosts': 'example.com:9200',
        })

    assert str(client) == 'foo @ my-cloud-id'
    assert client.hosts is None


def test_es_client_to_and_from_dict():
    mock_app = MagicMock(name='PeekApp')
    mock_app.config.as_bool = MagicMock(return_value=False)
    client = connect(mock_app, **{
        'username': 'foo',
        'password': 'password',
        'hosts': 'example.com:9200',
        'use_ssl': True,
    })

    d = client.to_dict()
    assert d['password'] is None

    with patch.dict(os.environ, {'PEEK_PASSWORD': 'password'}):
        assert client.to_dict() == EsClient.from_dict(mock_app, d).to_dict()


def test_refreshing_es_client_to_and_from_dict():
    mock_app = MagicMock(name='PeekApp')
    mock_app.config.as_bool = MagicMock(return_value=False)
    parent = connect(mock_app, **{
        'username': 'foo',
        'password': 'password',
        'hosts': 'example.com:9200',
        'use_ssl': True,
    })

    client = RefreshingEsClient(
        parent=parent,
        username='bar@example.com',
        access_token='access_token',
        refresh_token='refresh_token',
        expires_in=42,
        name='my-refreshing-client'
    )

    assert client.to_dict() == RefreshingEsClient.from_dict(client.to_dict()).to_dict()


@patch.dict(os.environ, {'PEEK_PASSWORD': 'password'})
def test_es_client_manager():
    mock_app = MagicMock(name='PeekApp')
    mock_app.config.as_bool = MagicMock(return_value=False)

    es_client_manager = EsClientManager()
    local_admin_0 = EsClient(name='local-admin', hosts='localhost:9200', username='admin', password='password')
    local_foo_1 = EsClient(name='local-foo', hosts='localhost:9200', username='foo', password='password')
    local_bar_saml_2 = RefreshingEsClient(parent=local_admin_0, username='bar@example.com', access_token='access_token',
                                          refresh_token='refresh_token', expires_in=42, name='local-bar-saml')
    remote_admin_3 = EsClient(name='remote-admin', hosts='example.com:9200', username='elastic', password='password')
    remote_oidc_4 = RefreshingEsClient(parent=EsClient(name='removed', hosts='example.com:9200'), username='dangling',
                                       access_token='access_token', refresh_token='refresh_token', expires_in=42,
                                       name='remote-dangling-oidc')

    es_client_manager.add(local_admin_0)
    es_client_manager.add(local_foo_1)
    es_client_manager.add(local_bar_saml_2)
    es_client_manager.add(remote_admin_3)
    es_client_manager.add(remote_oidc_4)

    assert remote_oidc_4 == es_client_manager.current

    es_client_manager.set_current(2)
    assert local_bar_saml_2 == es_client_manager.current

    es_client_manager.set_current('remote-admin')
    assert remote_admin_3 == es_client_manager.current

    assert local_foo_1 == es_client_manager.get_client(1)
    assert local_foo_1 == es_client_manager.get_client('local-foo')
    assert remote_admin_3 == es_client_manager.get_client(None)  # same as get current

    d = es_client_manager.to_dict()
    assert d == {'_index_current': 3, '_clients': [
        {'name': 'local-admin', 'hosts': 'localhost:9200', 'cloud_id': None, 'username': 'admin', 'password': None,
         'use_ssl': False, 'verify_certs': False, 'assert_hostname': False, 'ca_certs': None, 'client_cert': None,
         'client_key': None, 'api_key': None, 'token': None, 'headers': None},
        {'name': 'local-foo', 'hosts': 'localhost:9200', 'cloud_id': None, 'username': 'foo', 'password': None,
         'use_ssl': False, 'verify_certs': False, 'assert_hostname': False, 'ca_certs': None, 'client_cert': None,
         'client_key': None, 'api_key': None, 'token': None, 'headers': None},
        {'name': 'local-bar-saml', 'username': 'bar@example.com', 'access_token': 'access_token',
         'refresh_token': 'refresh_token', 'expires_in': 42, 'parent': 0},
        {'name': 'remote-admin', 'hosts': 'example.com:9200', 'cloud_id': None, 'username': 'elastic', 'password': None,
         'use_ssl': False, 'verify_certs': False, 'assert_hostname': False, 'ca_certs': None, 'client_cert': None,
         'client_key': None, 'api_key': None, 'token': None, 'headers': None},
        {'name': 'remote-dangling-oidc', 'username': 'dangling', 'access_token': 'access_token',
         'refresh_token': 'refresh_token', 'expires_in': 42,
         'parent': {'name': 'removed', 'hosts': 'example.com:9200', 'cloud_id': None, 'username': None,
                    'password': None, 'use_ssl': False, 'verify_certs': False, 'assert_hostname': False,
                    'ca_certs': None, 'client_cert': None, 'client_key': None, 'api_key': None, 'token': None,
                    'headers': None}}]}

    new_manager = EsClientManager.from_dict(mock_app, d)

    clients = new_manager.clients()
    assert len(clients) == 5
    assert clients.index(new_manager.current) == 3

    assert d == new_manager.to_dict()



#!/usr/bin/env python

"""Tests for `peek` package."""
import json
import os
from unittest.mock import patch, MagicMock

import pytest
from configobj import ConfigObj

from peek.common import AUTO_SAVE_NAME
from peek.peekapp import PeekApp


def test_multiple_stmts():
    text = """get abc

post abc/_doc
{ "foo":
         "bar"
}

conn foo=bar
get abc
"""
    nodes = []
    peek = PeekApp(batch_mode=True, extra_config_options=('log_level=None',))
    peek.execute_node = lambda stmt: nodes.append(stmt)
    peek.process_input(text)

    for n in nodes:
        print(n)

    assert [str(n) for n in nodes] == [
        r'''get abc {}
''',
        r'''post abc/_doc {}
{"foo":"bar"}
''',
        r'''conn [] [] {foo:bar}
''',
        r'''get abc {}
''',
    ]


@patch('peek.peekapp.PromptSession', MagicMock())
def test_app_will_not_auto_load_session_by_default(config_obj):
    mock_history = MagicMock()
    MockHistory = MagicMock(return_value=mock_history)

    def get_config(_, extra_config):
        config_obj.merge(ConfigObj(extra_config))
        return config_obj

    with patch('peek.peekapp.get_config', get_config), patch('peek.peekapp.SqLiteHistory', MockHistory):
        PeekApp(extra_config_options=('log_level=None', 'use_keyring=False'))

    mock_history.load_session.assert_not_called()


@patch('peek.peekapp.PromptSession', MagicMock())
def test_app_will_not_auto_load_session_if_connection_is_specified_on_cli(config_obj):
    config_obj['auto_load_session'] = True
    mock_history = MagicMock()
    MockHistory = MagicMock(return_value=mock_history)

    def get_config(_, extra_config):
        config_obj.merge(ConfigObj(extra_config))
        return config_obj

    class MockCliNs:

        def __init__(self):
            self.username = 'foo'
            self.password = 'password'

    with patch('peek.peekapp.get_config', get_config), patch('peek.peekapp.SqLiteHistory', MockHistory):
        PeekApp(extra_config_options=('log_level=None', 'use_keyring=False'), cli_ns=MockCliNs())

    mock_history.load_session.assert_not_called()


@patch('peek.peekapp.PromptSession', MagicMock())
def test_app_will_not_auto_load_session_if_connection_is_specified_in_rcfile(config_obj):
    config_obj['auto_load_session'] = True
    mock_history = MagicMock()
    MockHistory = MagicMock(return_value=mock_history)

    def get_config(_, extra_config):
        config_obj.merge(ConfigObj({
            'connection': {
                'username': 'foo',
                'password': 'password',
            }
        }))
        config_obj.merge(ConfigObj(extra_config))
        return config_obj

    with patch('peek.peekapp.get_config', get_config), patch('peek.peekapp.SqLiteHistory', MockHistory):
        PeekApp(extra_config_options=('log_level=None', 'use_keyring=False'))

    mock_history.load_session.assert_not_called()


@patch('peek.peekapp.PromptSession', MagicMock())
def test_app_will_auto_load_session_if_nothing_is_overriding_it(config_obj):
    config_obj['auto_load_session'] = True
    mock_history = MagicMock()
    mock_history.load_session = MagicMock(return_value=json.dumps(
        {'_index_current': 0, '_clients': [{'username': 'foo', 'password': 'bar', 'hosts': 'localhost:9200'}]}))
    MockHistory = MagicMock(return_value=mock_history)

    def get_config(_, extra_config):
        config_obj.merge(ConfigObj(extra_config))
        return config_obj

    with patch('peek.peekapp.get_config', get_config), patch('peek.peekapp.SqLiteHistory', MockHistory):
        PeekApp(extra_config_options=('log_level=None', 'use_keyring=False'))

    mock_history.load_session.assert_called_with(AUTO_SAVE_NAME)


@patch('peek.peekapp.PromptSession', MagicMock())
def test_app_will_auto_save_session(config_obj):
    mock_history = MagicMock()
    MockHistory = MagicMock(return_value=mock_history)

    def get_config(_, extra_config):
        config_obj.merge(ConfigObj(extra_config))
        return config_obj

    with patch('peek.peekapp.get_config', get_config), patch('peek.peekapp.SqLiteHistory', MockHistory):
        app = PeekApp(extra_config_options=('log_level=None', 'use_keyring=False'))
        app.signal_exit()
        app.run()

    mock_history.save_session.assert_called_once()


@patch('peek.peekapp.PromptSession', MagicMock())
def test_app_will_not_auto_save_session_when_disabled(config_obj):
    config_obj['auto_save_session'] = False
    mock_history = MagicMock()
    MockHistory = MagicMock(return_value=mock_history)

    def get_config(_, extra_config):
        config_obj.merge(ConfigObj(extra_config))
        return config_obj

    with patch('peek.peekapp.get_config', get_config), patch('peek.peekapp.SqLiteHistory', MockHistory):
        app = PeekApp(extra_config_options=('log_level=None', 'use_keyring=False'))
        app.signal_exit()
        app.run()

    mock_history.save_session.assert_not_called()


@pytest.fixture
def config_obj():
    from peek import __file__ as package_root
    package_root = os.path.dirname(package_root)
    package_config_file = os.path.join(package_root, 'peekrc')
    return ConfigObj(package_config_file)



import json
import os
from unittest.mock import MagicMock, patch

import pytest
from configobj import ConfigObj

from peek.connection import ConnectFunc
from peek.natives import ConnectionFunc, SessionFunc
from peek.peekapp import PeekApp

mock_history = MagicMock()
MockHistory = MagicMock(return_value=mock_history)


@pytest.fixture
def peek_app():
    from peek import __file__ as package_root
    package_root = os.path.dirname(package_root)
    package_config_file = os.path.join(package_root, 'peekrc')
    config_obj = ConfigObj(package_config_file)

    def get_config(_, extra_config):
        config_obj.merge(ConfigObj(extra_config))
        return config_obj

    class MockCliNs:

        def __init__(self):
            self.username = 'foo'
            self.password = 'password'

    with patch('peek.peekapp.PromptSession', MagicMock()), \
         patch('peek.peekapp.get_config', get_config), \
         patch('peek.peekapp.SqLiteHistory', MockHistory):
        return PeekApp(extra_config_options=('log_level=None', 'use_keyring=False'), cli_ns=MockCliNs())


@patch.dict(os.environ, {'PEEK_PASSWORD': 'password'})
def test_connection_related_funcs(peek_app):
    connect_f = ConnectFunc()
    assert '*  [1] bar @ https://localhost:9200' in connect_f(
        peek_app, username='bar', password='password', use_ssl=True)
    assert '*  [2] K-id @ http://example.com:9200' in connect_f(
        peek_app, api_key='id:key', hosts='example.com:9200')
    assert '*  [3] token-auth' in connect_f(
        peek_app, token='access_token', name='token-auth')

    connection_f = ConnectionFunc()
    assert connection_f(peek_app) == '''   [0] foo @ http://localhost:9200
   [1] bar @ https://localhost:9200
   [2] K-id @ http://example.com:9200
*  [3] token-auth'''

    assert connection_f(peek_app, 1) == '''   [0] foo @ http://localhost:9200
*  [1] bar @ https://localhost:9200
   [2] K-id @ http://example.com:9200
   [3] token-auth'''

    session_f = SessionFunc()
    assert "Session save as: '__default__'" == session_f(peek_app, **{'@': ['save']})
    mock_history.save_session.assert_called_with('__default__', json.dumps(peek_app.es_client_manager.to_dict()))
    mock_history.load_session = MagicMock(return_value=json.dumps(peek_app.es_client_manager.to_dict()))

    assert connection_f(peek_app, rename='local-bar') == '''   [0] foo @ http://localhost:9200
*  [1] local-bar
   [2] K-id @ http://example.com:9200
   [3] token-auth'''

    assert connection_f(peek_app, **{'@': ['info']}) == {
        'name': 'local-bar', 'hosts': 'localhost:9200',
        'cloud_id': None, 'auth': 'Username bar', 'use_ssl': True,
        'verify_certs': False, 'ca_certs': None, 'client_cert': None,
        'client_key': None, 'headers': None}

    assert connection_f(peek_app, remove=0) == '''*  [0] local-bar
   [1] K-id @ http://example.com:9200
   [2] token-auth'''

    assert connection_f(peek_app, 'token-auth') == '''   [0] local-bar
   [1] K-id @ http://example.com:9200
*  [2] token-auth'''

    assert connection_f(peek_app, keep=1) == '''*  [0] K-id @ http://example.com:9200'''

    assert session_f(peek_app, **{'@': ['load']}) == '''   [0] foo @ http://localhost:9200
*  [1] bar @ https://localhost:9200
   [2] K-id @ http://example.com:9200
   [3] token-auth'''
